/******************************************************************************/
/*                                                                            */
/*         Copyright(c) 2000-2002 by Ikanos Communications Inc.               */
/*                                                                            */
/*  All rights are reserved.  Reproduction in whole or in part is prohibited  */
/*  without the written consent of the copyright owner.                       */
/*                                                                            */
/*  The information and contents of this file are the proprietary information */
/*  of Ikanos Communication and may not be disclosed or used without the      */
/*  formal written approval of Ikanos Communication Inc.                      */
/*                                                                            */
/*  This Copyright notice may not be removed or modified without prior        */
/*  written consent of Ikanos Communications, Inc.                            */
/*                                                                            */
/******************************************************************************/
/*
**  Filename:    HostEnetPhyUtils.c
**
**  Project:     
**
**  Target:      MOTOROLA XPC855T based host board
**
**  Platform specific:
**    None.
**
**  Description:
**
**  $Revision: 1.1 $
**  $Date: 2006/10/16 04:34:51 $
**
**
**  **************************************************************************** 
**  Public Functions:
**
**  **************************************************************************** 
**  Private Functions:
**
**  **************************************************************************** 
**  Modification History:
**
**    Date         By     Description
**    -----------  ---    ------------------------------------------------------
**	  Jun-16-2002  SFA    Created
**    Jul-17-2002  SFA    Integration into new host code
**    Nov-19-2002  SFA    Ethernet PHY related function seperated from PTMF code
*/
/******************************************************************************/
/*------------------------------------------------------------------------------
  INCLUDE FILES
  ----------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>    

#include "dapi.h"
#include "apiosmacro.h"
#include "HostPtmfApi.h"
#include "HostEnetPhyUtils.h"

/*------------------------------------------------------------------------------
  PRIVATE PREPROCESSOR CONSTANTS
  ----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
  PRIVATE PREPROCESSOR MACROS
  ----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
  PRIVATE ENUMERATION DEFINITIONS
  ----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
  PRIVATE TYPE DEFINITIONS
  ----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
  PUBLIC VARIABLES DEFINITIONS
  ----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
  PRIVATE VARIABLES DEFINITIONS
  ----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
  PRIVATE FUNCTION PROTOTYPE
  ----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
  PUBLIC FUNCTION DEFINITIONS
  ----------------------------------------------------------------------------*/
/******************************************************************************
*                                                                             *
* Function.. : iposReadEnetPhyStatus                        	                *
*                                                   	                        *
* Purpose... :                     	                                          *
*									                                                            *
* Parameters : Ethernet PHY MDIO PHY base address of the port                 *
*                                                                             *
* Returns... : 16 bit Ethernet PHY status value                               *
*                                                                             *
******************************************************************************/

U16 iposReadEnetPhyStatus(U8 bEnetPhyMdioAddr)
{
  tsHostIniMdioAddrData tsIniReg;

  // Ethernet Port address set for access to the port specific MDIO registers.
  tsIniReg.uiIniPhyAddr = bEnetPhyMdioAddr;
  
  // Read the status register from the specified Enet PHY port.
  tsIniReg.uiIniRegAddr = ENET_PHY_STATUS_REG;
  iposMdioRead (&tsIniReg);

  return (tsIniReg.uiIniRdWrData);
}

/******************************************************************************
*                                                                             *
* Function.. : iposReadEnetPartnerAbility                    	                *
*                                                   	                        *
* Purpose... :                     	                                          *
*									                                                            *
* Parameters : Ethernet PHY MDIO PHY base address of the port                 *
*                                                                             *
* Returns... : 16 bit Ethernet Link Partner Abiltiy register value            *
*                                                                             *
******************************************************************************/

U16 iposReadEnetPartnerAbility(U8 bEnetPhyMdioAddr)
{
  tsHostIniMdioAddrData tsIniReg;

  // Ethernet Port address set for access to the port specific MDIO registers.
  tsIniReg.uiIniPhyAddr = bEnetPhyMdioAddr;
  
  // Read the status register from the specified Enet PHY port.
  tsIniReg.uiIniRegAddr = ENET_PHY_LINK_ABILITY;
  iposMdioRead (&tsIniReg);

  return (tsIniReg.uiIniRdWrData);
}

/******************************************************************************
*                                                                             *
* Function.. : iposEnetPhyHwReset                      	                      *
*                                                   	                        *
* Purpose... :                     	                                          *
*									                                                            *
* Parameters : None                                                           *
*                                                                             *
* Returns... : None                                                           *
*                                                                             *
******************************************************************************/
#if defined(CPE_HOSTLESS)
void iposEnetPhyHwReset(void)
{
  U8  bDelay;

#ifdef DEBUG_INI
  if (gbDebugLevel <= 2)
    printf("\n Ethernet PHY Hardware Reset");
#endif //DEBUG_INI

  EnetPhyHwReset();
}

#endif //#if defined(CPE_HOSTLESS)

/*------------------------------------------------------------------------------
                                 END OF FILE
------------------------------------------------------------------------------*/
